unit HeaderGipalsDelphi;
{ Header unit for Gipals32.dll.

 There are two possible ways to work with the library Gipals32.dll.

 1) First way is a linear program construction by the provided set of functions,
    such as specification of the variables and constraints count and assignment
    of the constraint matrix elements. This way provides a complete flexibility
    of the linear programming modeling.

 2) The second way is importing linear program from a text file in MPS format.
    It can be done by one function LoadFromMPS. This way can be suitable if there
    is a set of prepared MPS files that need to be calculated.


   Optimalon Software, 2005
}

interface

Const
  {Constraint sign}
  sign_Less  = 0; // <=
  sign_Equal = 1; // =
  sign_More  = 2; // >=

  {Preprocessor flags}
  prep_Disable          = $00; // Disable preprocessor
  prep_RemoveFixedVars  = $01; // Remove all fixed variables
  prep_RemoveEmptyCols  = $02; // Remove empty columns
  prep_RemoveSingular   = $04; // Remove singular rows and columns
  prep_RemoveLinearRows = $08; // Remove linear-dependent rows
  prep_RemoveLinearCols = $10; // Remove linear-dependent columns
  prep_ForcedRows       = $20; // Remove forced rows

  {Calculation result state (negative - unsuccessful, positive - successful)}
  calc_None          =  0; // Calculation has not been provided yet
  calc_Optimal       =  1; // Finished with optimal solution
  calc_Infeasible    = -1; // Failed with primal infeasibility
  calc_DblInfeasible = -2; // Failed with dual infeasibility
  calc_Unknown       =  2; // Finished because reached the max. of iterations
  calc_Suboptimal    =  3; // Finished due to slow convergence
  calc_Stoped        =  4; // Stoped by the user
  calc_Error         = -3; // Failed due to numerical eror
  calc_Unbounded     = -4; // Failed because the linear program is unbounded

  {Flags returned by the user-defined callback function}
  res_Abort = -1; // Calculation is canceled
  res_None  =  0; // Calculation continues
  res_Stop  =  1; // Calculation is stoped and the results are ready

  {The user-defined callback function must have the following format:

   Function CallBack(IterCount: Integer;
                     PrimalObj,
                     DualObj,
                     InfPrimal,
                     InfDual,
                     Optimality: Double): Integer; stdcall;
  }

// Set number of conastraints (rows)
Procedure SetConstraintCount(ACount: Integer); stdcall; external 'Gipals32.dll' name 'SetConstraintCount@4';

// Set number of decision (primal) variables
Procedure SetVariableCount(ACount: Integer); stdcall; external 'Gipals32.dll' name 'SetVariableCount@4';

// Returns number of constraints
Function GetConstraintCount: Integer; stdcall; external 'Gipals32.dll' name 'GetConstraintCount@0';

// Returns number of decision (primal) variables
Function GetVariableCount: Integer; stdcall; external 'Gipals32.dll' name 'GetVariableCount@0';

{ Clears the linear program. Deletes all existing data and set number of
   constraints and variables to 1 }
Procedure ClearLinearProgram; stdcall; external 'Gipals32.dll' name 'ClearLinearProgram@0';

// Set the objective to minimize (1) or maximize (0)
Procedure SetObjectiveType(Minimization: Integer); stdcall; external 'Gipals32.dll' name 'SetObjectiveType@4';

// Set the cost and bounds for the variable Index. Returns 0 if variable does not exist.
Function SetVariable(Index: Integer;   // Variable index
                     Cost,             // Objective function coefficient
                     Lower,            // Lower bound of variable (ignored if LowerInf <> 0)
                     Upper: Double;    // Upper bound of variable (ignored if UpperInf <> 0)
                     LowerInf,         // If <> 0 then indicates variable has no lower bound
                     UpperInf: Integer // If <> 0 then indicates variable has no upper bound
                     ): Integer; stdcall; external 'Gipals32.dll' name 'SetVariable@36';

{ Add a new variable and set the cost and bounds for the variable Index.
   Returns a new variable index  if succeeded and -1 if the maximum size reached. }
Function AddVariable(Cost,             // Objective function coefficient
                     Lower,            // Lower bound of variable (ignored if LowerInf <> 0)
                     Upper: Double;    // Upper bound of variable (ignored if UpperInf <> 0)
                     LowerInf,         // If <> 0 then indicates variable has no lower bound
                     UpperInf: Integer // If <> 0 then indicates variable has no upper bound
                     ): Integer; stdcall; external 'Gipals32.dll' name 'AddVariable@32';

{ Set the right side and the sign of the specified constraint Index.
   Returns 0 if constraint does not exist. }
Function SetConstraint(Index: Integer; // Index of the constraint
                       Right: Double;  // Right side of the constraint
                       Sign: Integer   // Sign (sign_Less, sign_Equal, sign_More)
                       ): Integer; stdcall; external 'Gipals32.dll' name 'SetConstraint@16';

{ Add a new constraint and set its right side and the sign.
   Returns index of the constraint or -1 if the maximum size reached.}
Function AddConstraint(Right: Double; // Right side of the constraint
                       Sign: Integer  // Sign (sign_Less, sign_Equal, sign_More)
                       ): Integer; stdcall; external 'Gipals32.dll' name 'AddConstraint@12';

{ Set the coefficient for the specified variable VarIndex in the specified
   constraint RowIndex. Returns 1 if succeeded. }
Function SetConstraintElement(RowIndex,          // constraint index
                              VarIndex: Integer; // variable index
                              Value: Double      // coefficient value
                              ): Integer; stdcall; external 'Gipals32.dll' name 'SetConstraintElement@16';


{ The following two routines allow to fill the constraint matrix directly
   with much higher speed than SetConstraintElement function }
// Selects the constraints to fill. Returns 0 if the constraints does not exist
Function DirectSelectConstraint(Index: Integer): Integer; stdcall; external 'Gipals32.dll' name 'DirectSelectConstraint@4';

{ Add a new constraint element in fast way. Variable index MUST be in an
   ascending order. Returns 1 if succeeded. }
Function DirectAddConstraintElement(Index: Integer; // variable index
                                    Value: Double   // variable coefficient
                                    ): Integer; stdcall; external 'Gipals32.dll' name 'DirectAddConstraintElement@12';

{ Load Linear program from the specified file in MPS format.
   Returns: 1 if succeeded.
            0 if the maximum allowed number of variables or constraint reached
           -1 if the file is in wrong format }
Function LoadFromMPS(AFile: PChar): Integer; stdcall; external 'Gipals32.dll' name 'LoadFromMPS@4';

{ The main function. Calculates the linear program and returns result.
   The result is defined in "Calculation result" section above.
   ProcessFunc is callbac function provided to control iteration and cancel
   or stop the calculation  }
Function Calculate(ProcessFunc: Pointer): Integer; stdcall; external 'Gipals32.dll' name 'Calculate@4';

// GET RESULTS
// Returns primal objective function value
Function PrimalObjective: Double; stdcall; external 'Gipals32.dll' name 'PrimalObjective@0';

// Returns dual objective function value
Function DualObjective: Double; stdcall; external 'Gipals32.dll' name 'DualObjective@0';

// Returns value of the specified decision (primal) variable (X)
Function PrimalValue(Index: Integer): Double; stdcall; external 'Gipals32.dll' name 'PrimalValue@4';

// Returns value of the specified dual variable (Y)
Function DualValue(Index: Integer): Double; stdcall; external 'Gipals32.dll' name 'DualValue@4';

// Returns value of the specified reduced cost variable (S)
Function ReducedCostValue(Index: Integer): Double; stdcall; external 'Gipals32.dll' name 'ReducedCostValue@4';


/////////////////////////////////////////////////////////////////////////////////////////////
// The following routines are optional and should be used to tune the calculation engine.  //
/////////////////////////////////////////////////////////////////////////////////////////////

// Flags are combination of Preprocessor flags or prep_Disable to disable
Procedure SetPreprocessor(Flags: Integer); stdcall; external 'Gipals32.dll' name 'SetPreprocessor@4';

// Set tolerances. Defaults are: Primal=1e-8, Dual=1e-8, Optimal=1e-10
Procedure SetTolerances(Primal,Dual,Optimal: Double); stdcall; external 'Gipals32.dll' name 'SetTolerances@24';

// Set maximum of iterations (default is 100)
Procedure SetMaxIterationCount(Value: Integer); stdcall; external 'Gipals32.dll' name 'SetMaxIterationCount@4';

// Enable/Disable the LP scaling. 1 - enable (default), 0 - disable.
Procedure SetScaling(Value: Integer); stdcall; external 'Gipals32.dll' name 'SetScaling@4';

// Enable/Disable the iterative refinement. 1 - enable (default), 0 - disable.
Procedure SetRefinement(Value: Integer); stdcall; external 'Gipals32.dll' name 'SetRefinement@4';

// Use the Gondzio correction. 1 - use (default), 0 - don't.
Procedure SetGondzio(Value: Integer); stdcall; external 'Gipals32.dll' name 'SetGondzio@4';


implementation

end.
